/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/platform/iphone/iphone_video.m $
 * $Id: iphone_video.m 44495 2009-09-30 16:16:53Z fingolfin $
 *
 */

//INCLUDES
#include <semaphore.h>
#include <fstream>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>
#include <vector>
#import <engines/engine.h>
#import <engines/igob/gob.h>
#import <engines/igob/goblin.h>
#import <engines/igob/script.h>
#import <engines/igob/hotspots.h>
#import <engines/igob/game.h>
#include "system.h"
#include "iphone_video.h"
#include "osys_main.h"
#import "menuigViewController.h"
#import "switchViewController.h"

//---VIDEO VIEW PARAMS
char* _textureBuffer = 0;
int _textureWidth = 0;
int _textureHeight = 0;
static int _width = 0;
static int _height = 0;
static int _needsScreenUpdate = 0;
static CGRect _screenRect;
static iPhoneView *sharedInstance = nil;
//---END

//////////////////
//---GAME PARAMS
//LVL (GENERAL)
extern bool bLoadingLevel;
extern bool bLvLLoaded;
extern int idLvLCourant;
//LVL (SIMULATION)
extern bool bSimulation;
extern bool bLoadSimulLaunch;
//GAME (GENERAL)
extern bool bPopup;
extern bool bMenuIngame;
extern bool bHintMenuActive;
extern bool bFromMenuIngame;
extern bool bAnimBeginning;
extern bool bMenuGob1;
extern bool bLoad;
extern bool bBackground;
extern int bGob2Action;
extern int counter;
extern int winkleLeft, winkleTop, fingusLeft, fingusTop, princeLeft, princeTop, princeLastLeft;
extern int numberGobliin;
extern bool changeWorld;

bool bEndGame = false;
bool bNewGame = false;
bool bTutoOn = false;
CGPoint dist;
CGPoint m;
//SOUND
extern int idSoundMusic;
extern int idSoundSFX;
extern bool bSoundMusicActive;
extern AVAudioPlayer* menuPlayer;
//DEFINES
extern switchViewController* g_switchVC;
extern OSystem* g_system;
extern std::vector< Gob::Hotspots::Hotspot* > _HotspotObj;
extern uint16* _fullscreen;
//---END
////////////////

extern bool bSem;
extern sem_t* sem;
extern int _GobLanguage;

int16 gob3State = -1;

//C-declarations
uint getSizeNextPOT(uint size);
//END

@implementation iPhoneView

+ (Class) layerClass
{
	return [CAEAGLLayer class];
}

- (id)initWithFrame:(struct CGRect)frame
{
	NSLog(@"iPhoneView initWithFrame");
	[super initWithFrame: frame];
	_fullWidth = frame.size.width;
	_fullHeight = frame.size.height;
	_screenLayer = nil;
	
	sharedInstance = self;
	
	_lock = [NSLock new];
	_context = nil;
	_screenTexture = 0;
	
	return self;
}

- (void)awakeFromNib
{
	NSLog(@"iPhoneView awakeFromNib");	
	_fullWidth = self.frame.size.width;
	_fullHeight = self.frame.size.height;
	_screenLayer = nil;
	
	sharedInstance = self;
	
	_lock = [NSLock new];
	_context = nil;
	_screenTexture = 0;
	
	for( int i = 0; i < 50; ++i )
	{
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
		alphaDisplayed[i] = false;		
	}
	iconsDisplayed = false;
	bGob2RightClick = false;
	bGob2Flag = false;
	hotspotCountLeftC = 0;
	hotspotCountRightC = 0;
	bFlagIsComputing = false;
	bFlagCleared = true;
	bObjectsToGet = true;
	
	mEtatAction = 0;
	stepPosX = 0.0f;
	stepPosY = 0.0f;
	bStepIconOn = false;
	
	activityIndicator.hidden = YES;

	//ingame menu buttons
	skipButton.hidden = YES;
	menuButton.hidden = YES;	
	hintButton.hidden = YES;	
	choiceButton.hidden = YES;
	rightClickButton.hidden = YES;
	gobliinButton.hidden = YES;
	openMenuBar.hidden = YES;
	closeMenuBar.hidden = YES;	
	
	skipButton.exclusiveTouch = YES;
	menuButton.exclusiveTouch = YES;	
	hintButton.exclusiveTouch = YES;	
	choiceButton.exclusiveTouch = YES;
	rightClickButton.exclusiveTouch = YES;
	gobliinButton.exclusiveTouch = YES;
	openMenuBar.exclusiveTouch = YES;
	closeMenuBar.exclusiveTouch = YES;		

	choiceButton.alpha = 1.0;	
	rightClickButton.alpha = 1.0;
	gobliinButton.alpha = 1.0;
	openMenuBar.alpha = 1.0;
	
	//touch notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(levelBeginNotification)  name:@"levelBeginNotif" object:nil];
	//clearAll notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(clearAllNotification)  name:@"clearAllNotif" object:nil];	
	//hide buttons notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(hideButtonsNotification)  name:@"hideButtonsNotif" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(skipButtonOnNotification)  name:@"skipButtonOnNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(skipButtonOffNotification)  name:@"skipButtonOffNotif" object:nil];
	//set sounds notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(setSoundsNotification)  name:@"setSoundsNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(muteNotification)  name:@"muteNotif" object:nil];	
	//level notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(lvlLoadedNotification)  name:@"lvlLoadedNotif" object:nil];
	//loading notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingScreenOnNotification)  name:@"loadingScreenOnNotif" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingScreenOffNotification)  name:@"loadingScreenOff" object:nil];		
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadingOffAnimBeginningNotification)  name:@"loadingOffAnimBeginningNotif" object:nil];
	//end game notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endGameNotification)  name:@"endGameNotif" object:nil];
	//load simul notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadSimulNotification)  name:@"loadSimulNotif" object:nil];
	//control mode notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(cursorModeNotification)  name:@"cursorModeNotif" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(touchModeNotification)  name:@"touchModeNotif" object:nil];
	//end fleche anim notif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endAnimFlecheNotification)  name:@"endAnimFlecheNotif" object:nil];	
	//subtitlesONNotif
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(subtitlesONNotification)  name:@"subtitlesONNotif" object:nil];	
	//switch cursor bag
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(switchCursorBagONNotification)  name:@"switchCursorBagONNotif" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(switchCursorBagOFFNotification)  name:@"switchCursorBagOFFNotif" object:nil];		
	//switch cursor door
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(switchCursorDoorNotification:)  name:@"switchCursorDoorNotif" object:nil];
	//reset pos to fingus
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(resetPosToFingusNotification:)  name:@"resetPosToFingusNotif" object:nil];	
	//end tuto
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(endTutoAgainNotification)  name:@"endTutoAgainNotif" object:nil];	
	//hide menu Bar
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(hideMenuBarNotification:)  name:@"hideMenuBarNotif" object:nil];		
}

-(void) writeSettingsSound
{
	NSLog(@"iPhoneView writeSettingsSound");		
    std::fstream settingsFile;
	
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];	
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
	
    settingsFile.open([path_sound UTF8String], std::fstream::out | std::fstream::app);
    if(!settingsFile.is_open())
    {
        ;//NSLog(@"Can't create/open file %@", path);
    }
	
	//idSoundSFX = g_system->getMixer()->getVolumeForSoundType(Audio::Mixer::kSFXSoundType);	
	
    NSString* infosTabSeparated;
	if( !bSoundMusicActive )
	{
		infosTabSeparated = [[NSString alloc] initWithFormat:@"music %i sfx %i    \t", 192, idSoundSFX];
	}		
	else 
		infosTabSeparated = [[NSString alloc] initWithFormat:@"music %i sfx %i    \t", idSoundMusic, idSoundSFX];		
	
    int buffLength = [infosTabSeparated length];
    char* buffer = new char[buffLength];
    [infosTabSeparated getBytes:(void*)buffer maxLength:buffLength usedLength:NULL encoding:NSUTF8StringEncoding options:NSStringEncodingConversionAllowLossy range:NSMakeRange(0, buffLength) remainingRange:NULL];
    settingsFile.write (buffer, buffLength);
    settingsFile.close();
    [infosTabSeparated release];
    delete[] buffer;
}

- (void)applicationSuspend 
{
	NSLog(@"iPhoneView applicationSuspend");			
	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    path_sound = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"SETTINGS_SOUND.DAT"]];
	
	NSFileManager* fileManager = [NSFileManager defaultManager];
	[fileManager removeItemAtPath:path_sound error:NULL];
	
	[self writeSettingsSound];
	
	NSData* someData2 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_SOUND.DAT"]];
	//NSLog([[[NSString alloc] initWithData:someData2 encoding:NSUTF8StringEncoding ] autorelease]);
	
	//UI
	[self hideMenuBar];	
}

///----NOTIFICATIONS

- (void)resetPosToFingusNotification:(NSNotification*)_notif
{
	[self performSelectorOnMainThread:@selector(resetPosToFingus) withObject: nil waitUntilDone:NO];
}

- (void)switchCursorDoorNotification:(NSNotification*)_notif
{
	[self performSelectorOnMainThread:@selector(switchCursorDoor) withObject: nil waitUntilDone:NO];
}

- (void)hideMenuBarNotification:(NSNotification*)_notif
{
	[self performSelectorOnMainThread:@selector(hideMenuBar) withObject: nil waitUntilDone:NO];
}

-(void)muteNotification
{
	[self performSelectorOnMainThread:@selector(mute) withObject: nil waitUntilDone:NO];
}

-(void)switchCursorBagONNotification
{
	[self performSelectorOnMainThread:@selector(switchCursorBagON) withObject: nil waitUntilDone:NO];
}

-(void)switchCursorBagOFFNotification
{
	[self performSelectorOnMainThread:@selector(switchCursorBagOFF) withObject: nil waitUntilDone:NO];	
}

-(void)loadingScreenOnNotification
{
	[self performSelectorOnMainThread:@selector(loadingScreenOn) withObject: nil waitUntilDone:NO];
}

-(void)loadingScreenOffNotification
{
	[self performSelectorOnMainThread:@selector(loadingScreenOff) withObject: nil waitUntilDone:NO];
}

-(void)loadingOffAnimBeginningNotification
{
	[self performSelectorOnMainThread:@selector(loadingOffAnimBeginning) withObject: nil waitUntilDone:NO];
}

-(void)setSoundsNotification
{
	[self performSelectorOnMainThread:@selector(setSounds) withObject: nil waitUntilDone:NO];
}

-(void)clearAllNotification
{
	[self performSelectorOnMainThread:@selector(clearAll) withObject: nil waitUntilDone:NO];
}

-(void)levelBeginNotification
{
	[self performSelectorOnMainThread:@selector(levelBegin) withObject: nil waitUntilDone:NO];
}

-(void)loadSimulNotification
{
	[self performSelectorOnMainThread:@selector(loadSimulWait) withObject: nil waitUntilDone:NO];		
}

-(void)touchModeNotification
{
	[self performSelectorOnMainThread:@selector(touchMode) withObject: nil waitUntilDone:NO];		
}

-(void)cursorModeNotification
{
	[self performSelectorOnMainThread:@selector(cursorMode) withObject: nil waitUntilDone:NO];		
}

-(void)lvlLoadedNotification
{
	[self performSelectorOnMainThread:@selector(levelLoaded) withObject: nil waitUntilDone:NO];	
}

-(void)hideButtonsNotification
{
	[self performSelectorOnMainThread:@selector(hideButtons) withObject: nil waitUntilDone:NO];	
}

-(void)skipButtonOnNotification
{
	[self performSelectorOnMainThread:@selector(skipButtonOn) withObject: nil waitUntilDone:NO];	
}

-(void)skipButtonOffNotification
{
	[self performSelectorOnMainThread:@selector(skipButtonOff) withObject: nil waitUntilDone:NO];	
}

-(void)endGameNotification
{
	[self performSelectorOnMainThread:@selector(endGame) withObject: nil waitUntilDone:NO];	
}

-(void)endAnimFlecheNotification
{
	[self performSelectorOnMainThread:@selector(endFlecheAnim) withObject: nil waitUntilDone:NO];	
}	

-(void)subtitlesONNotification
{
	[self performSelectorOnMainThread:@selector(subtitlesON) withObject: nil waitUntilDone:NO];	
}	

-(void)endTutoAgainNotification
{
	[self performSelectorOnMainThread:@selector(endTutoAgain) withObject: nil waitUntilDone:NO];	
}
///---END NOTIFICATIONS
-(void)muteAfterInit
{
	g_system->getMixer()->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, 0);
	g_system->getMixer()->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, 0);	
}

-(void)mute
{
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(muteAfterInit) userInfo:nil repeats:NO];	
}

-(void)subtitlesON
{
	if( counter != 28 && !bGob2RightClick && !bGob2Flag )
	{
		openMenuBar.hidden = YES;
		//choiceButton.hidden = YES;
		rightClickButton.hidden = YES;
		bSubs = true;
	}
}

-(void)loadingScreenOn
{
	NSLog(@"iPhoneView loadingScreenOn");
	
	for( int i = 0; i < 50; ++i )
	{
		hotspotImgView[i].hidden = YES;
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
		alphaDisplayed[i] = false;					
	}
	
	bLoadingImage = true;
	
	[loadingV removeFromSuperview];
	loadingV.hidden = YES;	
	
	if( loadingV.superview == nil )
	{
		[self addSubview:loadingV];
		loadingV.hidden = NO;
		loadingV.alpha = 0;
	}
	[self insertSubview:activityIndicator aboveSubview:loadingV];

	[UIView beginAnimations:nil context:self]; 
	{
		[UIView setAnimationDuration:0.05];
		[UIView setAnimationDelegate:self];
		[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
		loadingV.alpha = 1.0;
	}
	[UIView commitAnimations];
	
	rightClickButton.hidden = YES;
	gobliinButton.hidden = YES;
	openMenuBar.hidden = YES;
	cursorGobliiin.hidden = YES;
	
	[self setUserInteractionEnabled:NO];	
	[activityIndicator startAnimating];
	activityIndicator.hidden = NO;	
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"muteNotif" object:nil];
	if( menuPlayer != nil )
		[menuPlayer stop];
}

-(void)loadingScreenOff
{
	NSLog(@"iPhoneView loadingScreenOff");			
	if( loadingV != nil )
	{
		[activityIndicator stopAnimating];
		activityIndicator.hidden = YES;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:1];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationDidStopSelector:@selector(animationFinished:finished:context:)];
		}
		[UIView commitAnimations];
	}
	bLoadingImage = false;
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"setSoundsNotif" object:nil];
}

//anim finished
- (void)animationFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context 
{	
	[self setUserInteractionEnabled:YES];	
	[loadingV removeFromSuperview];
	loadingV.hidden = YES;	
}

-(void)lOAB
{
	[[NSNotificationCenter defaultCenter] postNotificationName:@"skipButtonOnNotif" object:nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"loadingScreenOff" object:nil];		
}

-(void)loadingOffAnimBeginning
{
	[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(lOAB) userInfo:nil repeats:NO];
}

-(void)setSounds
{
	NSLog(@"iPhoneView setSounds");		
	if( !bLoadingImage )
	{
		g_system->getMixer()->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, idSoundSFX);
		g_system->getMixer()->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, idSoundMusic);		
	}
}

-(void)clearAll
{
	[self clearButtons];
}

-(void)levelBegin
{
	NSLog(@"iPhoneView levelBegin");	
	[self setUserInteractionEnabled:YES];
	bLvLLoaded = false;
	bAnimBeginning = false;
	skipButton.hidden = YES;
	bMenuIngame = false;
		
	_HotspotObj.clear();	
	
	for( int i = 0; i < 50; ++i )
	{
		hotspotImgView[i].hidden = YES;
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
		alphaDisplayed[i] = false;					
	}
		
	[self writeSettingsSound];		
	
	NSData* someData3 = [[NSData alloc] initWithContentsOfFile:[docsDirectory stringByAppendingPathComponent:@"SETTINGS_SOUND.DAT"]];
	//NSLog(@"%@",[[[NSString alloc] initWithData:someData3 encoding:NSUTF8StringEncoding] autorelease]);
}

-(void)loadSimulWait
{
	[NSTimer scheduledTimerWithTimeInterval:8.5 target:self selector:@selector(loadSimul) userInfo:nil repeats:NO];	
}

-(void)loadSimul
{
	g_system->warpMouse(20,0);
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLoadSimulation], @"type",
									   [NSNumber numberWithFloat:0], @"x",
									   [NSNumber numberWithFloat:0], @"y",nil]];		
}

-(void) clearWarningBar
{
	_warningBar.hidden = YES;
	_warningLabel.hidden = YES;	
}

-(void) doClear
{
	if( cursorFade )
		cursorFade = false;
	cursorGobliiin.alpha = 1;
}

//anim alpha
-(void) beginAlpha
{
	if( cursorFade )
	{
		cursorGobliiin.alpha = 1;
		
		[UIView beginAnimations:nil context:nil]; 
		{
			[UIView setAnimationDuration:0.25];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaFinished:finished:context:)];
			cursorGobliiin.alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( cursorFade )
	{
		cursorGobliiin.alpha = 0.25;
		
		[UIView beginAnimations:nil context:nil]; 
		{
			[UIView setAnimationDuration:0.25];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaBeginFinished:finished:context:)];
			cursorGobliiin.alpha = 1.0;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( cursorFade )
	{
		[self beginAlpha];
	}
}

-(void)touchMode
{
	if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
	{
		[cursorGobliiin removeFromSuperview];
		cursorGobliiin = nil;
		cursorGobliiin.hidden = YES;
	}
}

-(void)cursorMode
{
	[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
}

-(void) cursorCurrentGobliiin:(int)xxx coordY:(int)yyy
{
	NSLog(@"iPhoneViewController cursorCurrentGobliiin");	
	if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
	{
		[cursorGobliiin removeFromSuperview];
		cursorGobliiin = nil;
	}
	
	cursorGobliiin = [[UIImageView alloc] initWithFrame:CGRectMake(xxx*480/320, yyy*320/200, 40, 40)];	
	UIImage* curs;
	
	switch( numAction )
	{
		case 0:
			curs = [[UIImage imageNamed:@"curpas.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		case 1:
			curs = [[UIImage imageNamed:@"curexclamation.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		case 2:
			curs = [[UIImage imageNamed:@"cursac.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
		case 3:
			curs = [[UIImage imageNamed:@"curdoor.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;		
		default:
			curs = [[UIImage imageNamed:@"curpas.png"] retain];		
			[cursorGobliiin setImage:curs];		
			[curs release];	
			break;
	}
	cursorGobliiin.hidden = NO;
	[self addSubview:cursorGobliiin];
}

-(void) majCursor
{
	UIImage* curs;
	switch( numAction )
	{
		case 0:
			curs = [[UIImage imageNamed:@"curpas.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;
			[curs release];	
			break;
		case 1:
			curs = [[UIImage imageNamed:@"curexclamation.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;
		case 2:
			curs = [[UIImage imageNamed:@"cursac.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;
		case 3:
			curs = [[UIImage imageNamed:@"curdoor.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;		
			[curs release];	
			break;	
		default:
			curs = [[UIImage imageNamed:@"curpas.png"] retain];		
			cursorGobliiin.image = nil;
			cursorGobliiin.image = curs;	
			[curs release];	
			break;
	}
	cursorGobliiin.hidden = NO;	
}

- (void) resetPosToFingus
{
	float xTouch = fingusLeft + 10;
	float yTouch = fingusTop + 10;		
	
	[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];

	if( yTouch <= 10 ) yTouch = 10;
	g_system->warpMouse( xTouch , yTouch );	

	cursorGobliiin.frame = CGRectMake(xTouch *480/320,yTouch *320/200, 40, 40);	
}

-(void)switchCursorDoor
{	
	numAction = 3;
	[self majCursor];
}

-(void)switchCursorBagON
{
	UIImage* curs = [[UIImage imageNamed:@"sacadosopen.png"] retain];		
	[rightClickButton setImage:curs forState:UIControlStateNormal];
	[curs release];	
	
	numAction = 2;
	[self majCursor];
}

-(void)switchCursorBagOFF
{
	UIImage* curs = [[UIImage imageNamed:@"sacadosclose.png"] retain];		
	[rightClickButton setImage:curs forState:UIControlStateNormal];
	[curs release];	

	numAction = 0;
	[self majCursor];
}

//anim flechealpha
-(void) beginFlecheAlpha:(bool)w
{
	if( bAnimatingFleche )
	{
		if( w )
		{
			if( flecheGobliin == nil )
			{
				flecheGobliin = [[UIImageView alloc] initWithFrame:CGRectMake( ( winkleLeft + 7 ) * 480 / 320, ( winkleTop - 20 ) * 320 / 200, 20, 20)];
				UIImage* fleche = [[UIImage imageNamed:@"fleche bleu.png"] retain];		
				[flecheGobliin setImage:fleche];		
				[fleche release];		
				[self addSubview:flecheGobliin];
			}
			else
				flecheGobliin.frame = CGRectMake( ( winkleLeft + 7 ) * 480 / 320, ( winkleTop - 20 ) * 320 / 200, 20, 20);
		}
		else
			flecheGobliin.frame = CGRectMake( ( fingusLeft + 3 ) * 480 / 320, ( fingusTop - 20 ) * 320 / 200, 20, 20);	
		
		flecheGobliin.alpha = 1;
		
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaFlecheFinished:finished:context:)];
			flecheGobliin.alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaFlecheFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingFleche )
	{
		flecheGobliin.alpha = 0.25;		
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(alphaFlecheBeginFinished:finished:context:)];
			flecheGobliin.alpha = 1.0;
		}
		[UIView commitAnimations];
	}
}

- (void)alphaFlecheBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingFleche )
		[self beginFlecheAlpha:flecheWinkle];
}

-(void)switchFlecheGobliin
{
	flecheWinkle = !flecheWinkle;
}

-(void)endFlecheAnim
{
	if( bAnimatingFleche )
	{
		bAnimatingFleche = false;
		[timerFleche invalidate];
		timerFleche = nil;
		
		if( flecheGobliin != nil && flecheGobliin.superview != nil )
		{
			[flecheGobliin removeFromSuperview];
			flecheGobliin = nil;
		}
	}
}

-(void)beginAnimationButtons:(CGRect)rect
{
	if( bAnimatingButtons )
	{
		if( pointerButton == nil )
		{
			pointerButton = [[UIImageView alloc] initWithFrame:rect];
			switch( etatTuto )
			{
				case 3: pointerButton.transform = CGAffineTransformMakeRotation(3*3.14/4); break;	
				case 7: pointerButton.transform = CGAffineTransformMakeRotation(5*3.14/4); break;
				case 5: pointerButton.transform = CGAffineTransformMakeRotation(7*3.14/4); break;	
				case 6: pointerButton.transform = CGAffineTransformMakeRotation(3.14/4); break;					
			}
			UIImage* f = [[UIImage imageNamed:@"fleche jaune.png"] retain];		
			[pointerButton setImage:f];		
			[f release];		
			[self addSubview:pointerButton];
		}
		
		switch( etatTuto )
		{
			case 3: gobliinButton.alpha = 1.0; break;
			case 5:	rightClickButton.alpha = 1.0; break;
			case 6: openMenuBar.alpha = 1.0; break;	
		}
		pointerButton.alpha = 1;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(animationButtonsFinished:finished:context:)];
			switch( etatTuto )
			{
				case 3: gobliinButton.alpha = 0.25; break;
				case 5:	rightClickButton.alpha = 0.25; break;
				case 6: openMenuBar.alpha = 0.25; break;	
			}
			pointerButton.alpha = 0.25;
		}
		[UIView commitAnimations];
	}
}

- (void)animationButtonsFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingButtons )
	{
		switch( etatTuto )
		{
			case 3: gobliinButton.alpha = 0.25; break;
			case 5:	rightClickButton.alpha = 0.25; break;
			case 6: openMenuBar.alpha = 0.25; break;	
		}	
		pointerButton.alpha = 0.25;
		[UIView beginAnimations:nil context:self]; 
		{
			[UIView setAnimationDuration:0.35];
			[UIView setAnimationDelegate:self];
			[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
			[UIView setAnimationDidStopSelector:@selector(animationButtonsBeginFinished:finished:context:)];
			switch( etatTuto )
			{
				case 3: gobliinButton.alpha = 1.0; break;
				case 5:	rightClickButton.alpha = 1.0; break;
				case 6: openMenuBar.alpha = 1.0; break;	
			}
			pointerButton.alpha = 1;
		}
		[UIView commitAnimations];
	}
}

- (void)animationButtonsBeginFinished:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context
{	
	if( bAnimatingButtons )
		[self beginAnimationButtons:pointerButtonRect];
}

-(void)tutoStepOne
{
	etatTuto = 0;
	
	//bar tuto
	tutoBar.hidden = NO;
	[tutoBar setUserInteractionEnabled: NO];
	
	//msg begin lvl
	if( _GobLanguage == 0 )
	{
		msgBegin = [[UITextView alloc ] initWithFrame:CGRectMake( 30, 43, 480, 100 )];
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:15];		
	}
	else if( _GobLanguage == 1 )
	{
		msgBegin = [[UITextView alloc ] initWithFrame:CGRectMake( 5, 43, 480, 100 )];		
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:14];		
	}
	msgBegin.backgroundColor = [UIColor clearColor];
	msgBegin.textColor = [UIColor whiteColor];

	msgBegin.scrollEnabled = NO;
	msgBegin.editable = NO;
	msgBegin.autoresizingMask = UIViewAutoresizingFlexibleHeight;
	msgBegin.text = NSLocalizedString(@"Tuto1Text", @"Drag your cursor on a gobliin and click to select him!");
	[self addSubview:msgBegin];
	[msgBegin setUserInteractionEnabled: NO];	
	
	//header text
	if( _GobLanguage == 0 )
		headerText = [[UITextView alloc ] initWithFrame:CGRectMake( 150, 293, 400, 100 )];
	else if( _GobLanguage == 1 )
		headerText = [[UITextView alloc ] initWithFrame:CGRectMake( 115, 293, 400, 100 )];		
	headerText.backgroundColor = [UIColor clearColor];
	headerText.textColor = [UIColor whiteColor];
	headerText.font = [UIFont fontWithName:@"Verdana" size:15];
	headerText.scrollEnabled = NO;
	headerText.editable = NO;
	headerText.autoresizingMask = UIViewAutoresizingFlexibleHeight;
	headerText.text = NSLocalizedString(@"TutoBottomText1", @"Do the action to continue.");
	[self addSubview:headerText];
	[headerText setUserInteractionEnabled: NO];
	
	//fleche selection gobliins
	flecheWinkle = true;
	bAnimatingFleche = true;
	[self beginFlecheAlpha:flecheWinkle];
	timerFleche = [NSTimer scheduledTimerWithTimeInterval:3.5 target:self selector:@selector(switchFlecheGobliin) userInfo:nil repeats:YES];
	next = true;
}

-(void)tutoStepTwo
{
	if( next )
	{
		next = false;
		//end tuto one
		[self endFlecheAnim];
		
		bMovementActive = true;
		etatTuto = 1;
		
		//msg begin lvl
		msgBegin.frame = CGRectMake( 40, 43, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:15];
		msgBegin.text = NSLocalizedString(@"Tuto2Text", @"Now try to move a gobliin by touching the screen");
	}
}

-(void)tutoStepThree
{
	if( next )
	{
		next = false;
		
		//end tuto one
		[self endFlecheAnim];
		
		bMovementActive = true;
		etatTuto = 2;
		
		//msg begin lvl
		msgBegin.frame = CGRectMake( 4.5, 36.25, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
		msgBegin.text = NSLocalizedString(@"Tuto3Text", @"Just drag the cursor wherever you want and click anywhere to move the gobliin");
	}
}

-(void)tutoStepFour
{
	etatTuto = 3;	
	
	//msg begin lvl
	if( _GobLanguage == 0 )
		msgBegin.frame = CGRectMake( 47, 43, 480, 100 );
	else if( _GobLanguage == 1 )
		msgBegin.frame = CGRectMake( 20, 43, 480, 100 );		
	msgBegin.font = [UIFont fontWithName:@"Verdana" size:13];
	msgBegin.text = NSLocalizedString(@"Tuto4Text", @"This button will help you change your gobliin directly.");//You can pause the game, save your progression and change the settings here";
//	"Vous pouvez mettre en pause, sauvegarder ou changer les options ici";
	[self insertSubview:gobliinButton aboveSubview:tutoBar];
	
	//header text
	if( _GobLanguage == 0 )
		headerText.frame = CGRectMake( 145, 293, 400, 100 );
	else if( _GobLanguage == 1 )
		headerText.frame = CGRectMake( 120, 293, 400, 100 );		
	headerText.font = [UIFont fontWithName:@"Verdana" size:15];
	headerText.text = NSLocalizedString(@"TutoBottomText2", @"Tap the screen to continue.");
	
	bAnimatingButtons = true;
	pointerButtonRect = CGRectMake(58, 20, 20, 20);
	[self beginAnimationButtons:pointerButtonRect];
	
	next = true;
}

-(void)animNextButton
{
	bAnimatingButtons = true;
	switch( etatTuto )
	{
		case 5: pointerButtonRect = CGRectMake(425, 255, 20, 20); break;	
		case 6: pointerButtonRect = CGRectMake(35, 255, 20, 20); break;
		case 7: pointerButtonRect = CGRectMake(370, 117, 20, 20); break;			
	}
	[self beginAnimationButtons:pointerButtonRect];	
	next = true;
}

-(void)tutoStepFive
{
	bMovementActive = false;	
	bAnimatingButtons = false;
	gobliinButton.alpha = 1.0;
	if( pointerButton != nil && pointerButton.superview != nil )
	{
		[pointerButton removeFromSuperview];
		pointerButton = nil;
	}
	etatTuto = 4;
	
	//msg begin lvl
	if( _GobLanguage == 0 )
		msgBegin.frame = CGRectMake( 50, 43, 480, 100 );
	else if( _GobLanguage == 1 )
		msgBegin.frame = CGRectMake( 40, 43, 480, 100 );
	msgBegin.font = [UIFont fontWithName:@"Verdana" size:15];
	msgBegin.text = NSLocalizedString(@"Tuto5Text", @"If you need some help, just look at the hints!");
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
}

-(void)tutoStepSix
{
	if( next )
	{
		bMovementActive = false;		
		next = false;
		bAnimatingButtons = false;
		gobliinButton.alpha = 1.0;
		[self insertSubview:gobliinButton belowSubview:tutoBar];
		
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		etatTuto = 5;
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 7, 43, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 10, 43, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:14];
		msgBegin.text = NSLocalizedString(@"Tuto6Text", @"This bag contains the objects you took. Click here to select one");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepSeven
{
	if( next )
	{
		next = false;
		bAnimatingButtons = false;
		rightClickButton.alpha = 1.0;
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		etatTuto = 6;
		
		//msg begin lvl
		if( _GobLanguage == 0 )
			msgBegin.frame = CGRectMake( 7, 35.6, 480, 100 );
		else if( _GobLanguage == 1 )
			msgBegin.frame = CGRectMake( 9, 35.6, 480, 100 );			
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:14];
		msgBegin.text = NSLocalizedString(@"Tuto7Text", @"Click on this button to open the menu bar and select different help or settings.");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)tutoStepEight
{
	if( next )
	{
		next = false;
		bAnimatingButtons = false;
		openMenuBar.alpha = 1.0;
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		etatTuto = 7;
		
		//msg begin lvl
		msgBegin.frame = CGRectMake( 8, 43, 480, 100 );
		msgBegin.font = [UIFont fontWithName:@"Verdana" size:14];
		msgBegin.text = NSLocalizedString(@"Tuto8Text", @"Now, try to pick up the bottle. Remember to look at the hints!");
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(animNextButton) userInfo:nil repeats:NO];
	}
}

-(void)endTuto
{
	if( next )
	{
		next = false;
		bAnimatingButtons = false;	
		bTutoOn = false;
		bMovementActive = true;
		if( bSem )
		{
			sem_post( sem );		
			bSem = false;
		}
		if( flecheGobliin != nil && flecheGobliin.superview != nil )
		{
			[flecheGobliin removeFromSuperview];
			flecheGobliin = nil;
		}
		if( pointerButton != nil && pointerButton.superview != nil )
		{
			[pointerButton removeFromSuperview];
			pointerButton = nil;
		}
		if( tutoBar != nil && tutoBar.superview != nil )
		{
			tutoBar.hidden = YES;
		}
		if( msgBegin != nil && msgBegin.superview != nil )
		{
			[msgBegin removeFromSuperview];
			msgBegin = nil;
		}
		if( headerText != nil && headerText.superview != nil )
		{
			[headerText removeFromSuperview];
			headerText = nil;
		}
	}
}

-(void)endTutoAgain
{
	//NSLog(@"endTutoAgain");
	next = true;
	[self endTuto];
}

-(void) offLoading
{
	//buttons
	//menuButton.hidden = NO;		
	//hintButton.hidden = NO;
	//choiceButton.hidden = NO;	

	changeWorld = false;
	rightClickButton.hidden = NO;
	gobliinButton.hidden = NO;
	NSLog(@"/*-/*-/*--* %d,",numberGobliin);
	numberGobliin = 1;
	[self resetPosToFingus];
	
	if( idLvLCourant == 600 )
		gobliinButton.frame = CGRectMake( 420, 0, 60, 60 );
	else
		gobliinButton.frame = CGRectMake( 0, 0, 60, 60 );	

	[self hideMenuBar];
	
	if( idLvLCourant == 130 || idLvLCourant == 240 || idLvLCourant == 720 ) 
		gobliinButton.alpha = 0.7;
	else if( idLvLCourant == 420 || idLvLCourant == 430 || idLvLCourant == 540 || idLvLCourant == 560 )
		gobliinButton.alpha = 0.5;		
	else
		gobliinButton.alpha = 1.0;
	
	//cursor mouse
	reset = false;
	((OSystem_IPHONE*)g_system)->_mouseY = 125;	
	[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
	
	//go
	bMovementActive = true;
	
	//TUTO
	if( idLvLCourant == 110 && bNewGame )
	{	
		bNewGame = false;
		bTutoOn = true;
		[self tutoStepOne];
	}
	else if( !bNewGame )
	{
		[self endFlecheAnim];
		bTutoOn = false;
		[self endTuto];	
	}
	
	if( idLvLCourant == 280 )
	{
		//orientation
		if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeLeft && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft )
			[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
		else if( [[UIDevice currentDevice] orientation] == UIDeviceOrientationLandscapeRight && [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight )
			[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];

		[[BulkyServicesView sharedInstance] promptRating:YES];
	}
	
	[[NSNotificationCenter defaultCenter] postNotificationName:@"setSoundsNotif" object:nil];
}

-(void)levelLoaded
{
	NSLog(@"iPhoneView levelLoaded");	
	
	if( !bLoadSimulLaunch )
	{
		bFromMenuIngame = false;
		bLvLLoaded = true;
		bLoad = false;
		
		bGob2RightClick = false;
		bGob2Flag = false;
		
		if( idLvLCourant == 110 && bNewGame )
		{
			[NSTimer scheduledTimerWithTimeInterval:2.5 target:self selector:@selector(loadingScreenOff) userInfo:nil repeats:NO];	
			if( [[UIDevice currentDevice].systemVersion floatValue] > 3.2 )
				[NSTimer scheduledTimerWithTimeInterval:6.5 target:self selector:@selector(offLoading) userInfo:nil repeats:NO];
			else
				[NSTimer scheduledTimerWithTimeInterval:8.5 target:self selector:@selector(offLoading) userInfo:nil repeats:NO];
		}
		else
		{
			[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(loadingScreenOff) userInfo:nil repeats:NO];	
			[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(offLoading) userInfo:nil repeats:NO];
		}
	}
}

-(void)hideButtons
{
	NSLog(@"iPhoneView hideButtons");
	
	//buttons
	menuButton.hidden = YES;
	hintButton.hidden = YES;	
	choiceButton.hidden = YES;	
	rightClickButton.hidden = YES;
	gobliinButton.hidden = YES;	
	cursorGobliiin.hidden = YES;
	openMenuBar.hidden = YES;
	closeMenuBar.hidden = YES;
}

-(void)tempoSkip
{
	skipButton.hidden = NO;
	[self setUserInteractionEnabled:YES];
}

-(void)skipButtonOn
{
	//NSLog(@"iPhoneView skipButtonOn");
	[NSTimer scheduledTimerWithTimeInterval:1.5 target:self selector:@selector(tempoSkip) userInfo:nil repeats:NO];	
}

-(void)skipButtonOff
{
	//NSLog(@"iPhoneView skipButtonOff");
	skipButton.hidden = YES;
}

-(void)endAnimEndGame
{
	NSLog(@"iPhoneView endAnimEndGame");	
	bMenuIngame = false;
	bFromMenuIngame = true;
	bMenuGob1 = true;

	[g_switchVC switchViews];
	
	bSimulation = true;
	bLoadSimulLaunch = true;
	bEndGame = true;
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex 
{

}

-(void)endGame
{
	NSLog(@"iPhoneView endGame");
	
	//popup
	bPopup = true;
	
	//buttons
	[self hideButtons];
	
	//clear everything
	[self clearButtons];	
	
	//popup
	[[[UIAlertView alloc] initWithTitle:NSLocalizedString(@"EndGameTitle", @"Game Over!") message:NSLocalizedString(@"EndGameText", @"Congratulations! You completed the game and save the Prince Buffoon!") delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil] show];	


	[NSTimer scheduledTimerWithTimeInterval:30 target:self selector:@selector(endAnimEndGame) userInfo:nil repeats:NO];	//need to be < sur iPhone4	
}
//END NOTIF

-(void) dealloc
{
	NSLog(@"iPhoneView dealloc");
	if( skipButton != nil )
	{
		[skipButton removeFromSuperview];
		[skipButton release];
		skipButton = nil;
	}
	if( choiceButton != nil )
	{
		[choiceButton removeFromSuperview];
		[choiceButton release];
		choiceButton = nil;
	}
	if( hintButton != nil )
	{
		[hintButton removeFromSuperview];
		[hintButton release];
		hintButton = nil;
	}
	if( menuButton != nil )
	{
		[menuButton removeFromSuperview];
		[menuButton release];
		menuButton = nil;
	}
	if( rightClickButton != nil )
	{
		[rightClickButton removeFromSuperview];
		[rightClickButton release];
		rightClickButton = nil;
	}
	if( gobliinButton != nil )
	{
		[gobliinButton removeFromSuperview];
		[gobliinButton release];
		gobliinButton = nil;
	}
	if( openMenuBar != nil )
	{
		[openMenuBar removeFromSuperview];
		[openMenuBar release];
		openMenuBar = nil;
	}
	if( closeMenuBar != nil )
	{
		[closeMenuBar removeFromSuperview];
		[closeMenuBar release];
		closeMenuBar = nil;
	}
	
	if( tutoBar != nil )
	{
		[tutoBar removeFromSuperview];
		[tutoBar release];
		tutoBar = nil;
	}
	if( menuBar != nil )
	{
		[menuBar removeFromSuperview];
		[menuBar release];
		menuBar = nil;
	}
	
	if( flagAct != nil )
	{
		[flagAct removeFromSuperview];
		[flagAct release];
		flagAct = nil;
	}
	
	for( int i = 0; i < 50; ++i )
	{
		if( hotspotImgView[i] != nil )
		{
			[hotspotImgView[i] removeFromSuperview];
			[hotspotImgView[i] release];
			hotspotImgView[i] = nil;
		}
	}
	if( hotspotImg != nil )
	{
		[hotspotImg release];
		hotspotImg = nil;
	}
	if( invScreen != nil )
	{
		[invScreen release];
		invScreen = nil;
	}
	if( invView != nil )
	{
		[invView removeFromSuperview];
		[invView release];
		invView = nil;
	}
	if( alert != nil )
	{
		[alert release];
		alert = nil;
	}
	
	for( int i = 0; i < 5; ++i )
	{
		if( _flagLabel[i] != nil )
		{
			[_flagLabel[i] removeFromSuperview];
			[_flagLabel[i] release];
			_flagLabel[i] = nil;
		}
		
		if( _flagText[i] != nil )
		{
			[_flagText[i] removeFromSuperview];
			[_flagText[i] release];
			_flagText[i] = nil;
		}
	}
	
	if( _flagBackground != nil )
	{
		[_flagBackground removeFromSuperview];
		[_flagBackground release];
		_flagBackground = nil;
	}

	if( cursorGobliiin != nil )
	{
		[cursorGobliiin removeFromSuperview];
		[cursorGobliiin release];
		cursorGobliiin = nil;
	}
	
	if( timerFleche != nil )
	{
		[timerFleche invalidate];
		timerFleche = nil;
	}
	if( flecheGobliin != nil )
	{
		[flecheGobliin removeFromSuperview];
		[flecheGobliin release];
		flecheGobliin = nil;
	}
	if( pointerButton != nil )
	{
		[pointerButton removeFromSuperview];
		[pointerButton release];
		pointerButton = nil;
	}
	if( msgBegin != nil )
	{
		[msgBegin release];
		msgBegin = nil;
	}
	if( headerText != nil )
	{
		[headerText release];
		headerText = nil;
	}
	
	docsDirectory = nil;
	path = nil;
	path_sound = nil;
	path_control = nil;
	path_inv = nil;
	
	if( gameoverV != nil )
	{
		[gameoverV removeFromSuperview];
		[gameoverV release];
		gameoverV = nil;
	}
	
	if( loadingV != nil )
	{
		[loadingV removeFromSuperview];
		[loadingV release];
		loadingV = nil;
	}
	
	if( activityIndicator != nil )
	{
		[activityIndicator release];
		activityIndicator = nil;
	}

	//game
	delete gg;

	//opengl
	_screenLayer = nil;
	sharedInstance = nil;
	_context = nil;
	_screenTexture = 0;

	if (_screenTexture)
		free(_textureBuffer);
	
	[super dealloc];	
}

- (void *)getSurface 
{
	return _screenSurface;
}

- (void)updateSurface 
{
	if( !bBackground )
	{
		if (!_needsScreenUpdate) 
		{
			return;
		}
		_needsScreenUpdate = 0;
		
		GLfloat vertices[] =
		{
			0.0f + _heightOffset, 0.0f + _widthOffset,
			_visibleWidth - _heightOffset, 0.0f + _widthOffset,
			0.0f + _heightOffset,  _visibleHeight - _widthOffset,
			_visibleWidth - _heightOffset,  _visibleHeight - _widthOffset
		};
		
		float texWidth = _width / (float)_textureWidth;
		float texHeight = _height / (float)_textureHeight;
		
		const GLfloat texCoords[] =
		{
			texWidth, 0.0f,
			0.0f, 0.0f,
			texWidth, texHeight,
			0.0f, texHeight
		};
		
		glVertexPointer(2, GL_FLOAT, 0, vertices);
		glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
		
		if( !bSimulation ) //si on simule pas un niveau, on dessine, sinon on dessine rien
		{
			//[_lock lock];
			// Unfortunately we have to update the whole texture every frame, since glTexSubImage2D is actually slower in all cases
			// due to the iPhone internals having to convert the whole texture back from its internal format when used.
			// In the future we could use several tiled textures instead.
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, _textureWidth, _textureHeight, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, _textureBuffer);
			//[_lock unlock];
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
			[_context presentRenderbuffer:GL_RENDERBUFFER_OES];
		}
	}
}

- (void)initSurface
{
	NSLog(@"iPhoneView initSurface");		
	_textureWidth = getSizeNextPOT(_width);
	_textureHeight = getSizeNextPOT(_height);
	
	_screenOrientation = UIDeviceOrientationLandscapeRight;
	
	//printf("Window: (%d, %d), Surface: (%d, %d), Texture(%d, %d)\n", _fullWidth, _fullHeight, _width, _height, _textureWidth, _textureHeight);
	
	if (_context == nil)
	{
		_screenOrientation = UIDeviceOrientationLandscapeRight;
		CAEAGLLayer *eaglLayer = (CAEAGLLayer*) self.layer;
		
		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithBool:FALSE], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGB565, kEAGLDrawablePropertyColorFormat, nil];
		
		_context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
		if (!_context || [EAGLContext setCurrentContext:_context])
		{
			glGenFramebuffersOES(1, &_viewFramebuffer);
			glGenRenderbuffersOES(1, &_viewRenderbuffer);
			
			glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
			glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
			[_context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(id<EAGLDrawable>)self.layer];
			glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, _viewRenderbuffer);
			
			glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &_backingWidth);
			glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &_backingHeight);
			
			//printf(" backing %d; %d \n", _backingWidth, _backingHeight );
			
			if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) 
			{
				//NSLog(@"Failed to make complete framebuffer object %x.", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
				return;
			}
			
			_backingWidth = 480;
			_backingHeight = 600;
			glViewport(0, -140, _backingWidth, _backingHeight);
			//printf("glViewport - 0 0 %d %d\n", _backingWidth, _backingHeight);
			glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
			
			glEnable(GL_TEXTURE_2D);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glEnableClientState(GL_VERTEX_ARRAY);
		}
	}
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	if (_screenOrientation ==  UIDeviceOrientationLandscapeRight)
	{
		glRotatef(-180, 0, 0, 1); //-90
	} 
	else if (_screenOrientation == UIDeviceOrientationLandscapeLeft) 
	{
		glRotatef(180, 0, 0, 1); //90
	} 
	else 
	{
		glRotatef(-180, 0, 0, 1); //-180
	}
	
	glOrthof(0, _backingWidth, 0, _backingHeight, 0, 1);
	//printf("glOrthof - 0 %d 0 %d 0 1\n", _backingWidth, _backingHeight);
	
	if (_screenTexture > 0)
	{
		glDeleteTextures(1, &_screenTexture);
	}
	
	glGenTextures(1, &_screenTexture);
	glBindTexture(GL_TEXTURE_2D, _screenTexture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
	if (_textureBuffer) 
	{
		free(_textureBuffer);
	}
	
	int textureSize = _textureWidth * _textureHeight * 2;
	_textureBuffer = (char*)malloc(textureSize);
	memset(_textureBuffer, 0, textureSize);
	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
	
	// The color buffer is triple-buffered, so we clear it multiple times right away to avid doing any glClears later.
	//int clearCount = 5;
	//while (clearCount-- > 0)
	//{
		glClear(GL_COLOR_BUFFER_BIT);
		[_context presentRenderbuffer:GL_RENDERBUFFER_OES];
	//}
	
	if (_screenOrientation == UIDeviceOrientationLandscapeLeft || _screenOrientation ==  UIDeviceOrientationLandscapeRight)
	{
		_visibleHeight = _backingHeight;
		_visibleWidth = _backingWidth;
		
		float ratioDifference = ((float)_height / (float)_width) / ((float)_fullWidth / (float)_fullHeight);
		int rectWidth, rectHeight;
		
		if (ratioDifference < 1.0f) 
		{
			rectWidth = _fullWidth * ratioDifference;
			rectHeight = _fullHeight;
			_widthOffset = (_fullWidth - rectWidth) / 2;
			_heightOffset = 0;
		} 
		else 
		{
			rectWidth = _fullWidth;
			rectHeight = _fullHeight / ratioDifference;
			_heightOffset = (_fullHeight - rectHeight) / 2;
			_widthOffset = 0;
		}
		
		//printf("Rect: %i, %i, %i, %i\n", _widthOffset, _heightOffset, rectWidth, rectHeight);
		_screenRect = CGRectMake(_widthOffset, _heightOffset, rectWidth, rectHeight);
	} 
	else
	{
		float ratio = (float)_height / (float)_width;
		int height = _fullWidth * ratio;
		//printf("Making rect (%u, %u)\n", _fullWidth, height);
		_screenRect = CGRectMake(0, 0, _fullWidth - 1, height - 1);
		
		_visibleHeight = height;
		_visibleWidth = _backingWidth;
		_heightOffset = 0.0f;
		_widthOffset = 0.0f;
		
		CGRect keyFrame = CGRectMake(0.0f, 0.0f, 0.0f, 0.0f);
	}
}

- (id)getEvent
{	
	if (_events == nil || [_events count] == 0) 
	{
		return nil;
	}
	
	id event = [_events objectAtIndex: 0];
	
	[_events removeObjectAtIndex: 0];
	
	return event;
}

- (void)addEvent:(NSDictionary*)event 
{	
	if (_events == nil)
		_events = [[NSMutableArray alloc] init];
	
	[_events addObject: event];
}

- (void)deviceOrientationChanged:(UIDeviceOrientation)orientation 
{	
	if( orientation == UIDeviceOrientationLandscapeLeft )
	{
		_screenOrientation = orientation;
		//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeLeft animated:NO];
	}
	else if( orientation == UIDeviceOrientationLandscapeRight )
	{
		_screenOrientation = orientation;
		//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];	
	}
	else 
	{
		if( orientation == UIDeviceOrientationPortrait )
		{
			_screenOrientation = UIDeviceOrientationLandscapeRight;
			//[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
		}
	}
}

- (int)hotspotDistance:(CGPoint)point
{	
	float hotspotCenterX;
	float hotspotCenterY;
	int distX[20] = { 0 };
	int distY[20] = { 0 };
	
	//get the dist between the touch point and the center of the hotspot for each one
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
		hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;		
		distX[i] = abs( point.x - hotspotCenterX*320/480 );
		distY[i] = abs( point.y - hotspotCenterY*200/320 );
	}	
	int minx = 10, miny = 10, ind = -1;
	//we compare then with a minima and we return the one we are in
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		if( distX[i] < minx && distY[i] < miny )
		{
			minx = distX[i];
			miny = distY[i];
			ind = i;
		}
	}
	
	return ind;
}

- (void)hotspotDisplayCompute:(CGPoint)point
{	
	float hotspotCenterX;
	float hotspotCenterY;
	
	int ind = [self hotspotDistance:point];
	
	//PARCOURS DES HOTSPOTS
	for( int i = 0; i < _HotspotObj.size(); ++i )
	{
		hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
		hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;
		
		//GESTION AFFICHAGE DES "!"
		//look if the touch position is in the hotspot
		if( ind != -1 )
		{			
			if( numAction != 3 )
				numAction = 1;
			[self majCursor];
			bActionLocked = true;
			
			//clear all the other hotspots
			for( int j = 0; j < _HotspotObj.size(); ++j )
			{
				if( j != ind )
				{
					alphaDisplayed[j] = false;
					hotspotDisplayed[j] = false;
					hotspotImgView[j].hidden = YES;
					hotspotUsed[j] = false;
				}
			}
			
			hotspotCenterX = (_HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2)*480/320;
			hotspotCenterY = (_HotspotObj[ind]->top  + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2)*320/200;
			
			if( !hotspotDisplayed[ind] ) //if ! no displayed for this hotspot
			{				
				if( hotspotImgView[ind] == nil )
				{
					//alloc image view & image
					UIImageView* iv = [[UIImageView alloc] initWithFrame:CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 10, 10)];		
					hotspotImg = [[UIImage imageNamed:@"rond bleu.png"] retain];		
					[iv setImage:hotspotImg];		
					[hotspotImg release];		
					[self addSubview:iv];		
					
					hotspotImgView[ind] = iv;
					[iv release];
				}
				else
				{
					hotspotImgView[ind].hidden = NO;
					hotspotImgView[ind].frame = CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 10, 10);
				}
				hotspotDisplayed[ind] = true;
			}
			
			/*if( cursorGobliiin != nil && cursorGobliiin.superview != nil )
			{
				cursorGobliiin.hidden = YES;
			}*/
			
			//the icons are now displayed so don't bother do it again / hotspot used
			hotspotUsed[ind] = true;
			/*alphaDisplayed[ind] = false;
			hotspotImgView[ind].alpha = 1;
			hotspotImgView[ind].hidden = NO;*/
		}
		else
		//vision radius
		if( point.x > _HotspotObj[i]->left - 35 && point.x <= _HotspotObj[i]->right + 35 && point.y > _HotspotObj[i]->top - 35 && point.y <= _HotspotObj[i]->bottom + 35 )
		{			
			hotspotCenterX = (_HotspotObj[i]->left + (_HotspotObj[i]->right - _HotspotObj[i]->left)/2)*480/320;
			hotspotCenterY = (_HotspotObj[i]->top  + (_HotspotObj[i]->bottom - _HotspotObj[i]->top)/2)*320/200;
			
			if( !hotspotDisplayed[i] ) //if ! no displayed for this hotspot
			{				
				if( hotspotImgView[i] == nil )
				{
					//alloc image view & image
					UIImageView* iv = [[UIImageView alloc] initWithFrame:CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 10, 10)];		
					hotspotImg = [[UIImage imageNamed:@"rond bleu.png"] retain];		
					[iv setImage:hotspotImg];		
					[hotspotImg release];		
					[self addSubview:iv];		
					
					hotspotImgView[i] = iv;
					[iv release];
				}
				else
				{
					hotspotImgView[i].hidden = NO;
					hotspotImgView[i].frame = CGRectMake(hotspotCenterX - 5, hotspotCenterY - 5, 10, 10);
				}
				hotspotDisplayed[i] = true;
			}
			
			hotspotUsed[i] = false;
			hotspotUsed[ind] = false;
			
			/*if( bActionLocked && bGob2Action == 0 )
			{
				bActionLocked = false;
				numAction = 0;
				[self majCursor];
			}
			else if( bActionLocked && bGob2Action == 1 )
			{
				bActionLocked = false;
				numAction = 2;
				[self majCursor];	
			}*/
		}
		else //clear mode
		{			
			alphaDisplayed[i] = false;		
			hotspotImgView[i].hidden = YES;			
			hotspotDisplayed[i] = false;
			hotspotUsed[i] = false;
			
			if( ind != -1 )
			{
				alphaDisplayed[ind] = false;		
				hotspotImgView[ind].hidden = YES;			
				hotspotDisplayed[ind] = false;
				hotspotUsed[ind] = false;					
			}
			
			if( bActionLocked && bGob2Action == 0 )
			{
				bActionLocked = false;
				numAction = 0;
				[self majCursor];
			}
			else if( bActionLocked && bGob2Action == 1 )
			{
				bActionLocked = false;
				numAction = 2;
				[self majCursor];	
			}
			else if( bActionLocked && bGob2Action == 2 )
			{
				bActionLocked = false;
				numAction = 3;
				[self majCursor];	
			}
		}
	}	
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{	
	printf("touchesBegan !bMenuGob1 %d,!bPopup %d,bLvLLoaded %d,!bMenuIngame %d,!bHintMenuActive %d,!bSimulation %d,!bAnimBeginning %d, !bLoadingImage %d\n", 
		   bMenuGob1,bPopup,bLvLLoaded,bMenuIngame,bHintMenuActive,bSimulation,bAnimBeginning, bLoadingImage);
		
	if( changeWorld || bNewGame ) 
		return;
	
	/*if( !bFlagCleared && !bFlagIsComputing )
		[self removeBoxFlag];*/
	
	//[self removeObjectsToGetButtons];
	
	if( !bMenuGob1 && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning && !bLoadingImage && !bFlagIsComputing )
	{
		[self hideMenuBar];	
		
		UITouch *touch = [touches anyObject];
		CGPoint point = [touch locationInView:self];
			
		//gestion ST
		if( bSubs )
		{
			//openMenuBar.hidden = NO;
			//choiceButton.hidden = NO;
			rightClickButton.hidden = NO;
			bSubs = false;
		}
				
		//GESTION TOUCH
		if( point.y < 5 )
			point.y = 5;
		else if( point.y > 256 )
			point.y = 256;
		
		//Si mode cursor
		if( bMovedCursor )
		{
			bMovedCursor = false;
			point.x = point.x * 320 / 480;
			point.y = point.y * 200 / 320;
			
			dist.x = ((OSystem_IPHONE*)g_system)->_mouseX - point.x;
			dist.y = ((OSystem_IPHONE*)g_system)->_mouseY - point.y;	
			
			switch( etatTuto )
			{
				case 2:	[self tutoStepFour]; break;
			}
			
		}
		//mode touch
		else
		{			
			point.x = point.x * 320 / 480;
			point.y = point.y * 200 / 320;
			
			dist.x = ((OSystem_IPHONE*)g_system)->_mouseX - point.x;
			dist.y = ((OSystem_IPHONE*)g_system)->_mouseY - point.y;
			
			/*if( cursorGobliiin == nil )
				[self cursorCurrentGobliiin:point.x coordY:point.y];
			else
				cursorGobliiin.frame = CGRectMake(point.x *480/320,point.y *320/200, 40, 40);		
			cursorGobliiin.hidden = NO;
			
			g_system->warpMouse(point.x, point.y);*/
			
			//TUTO
			if( bTutoOn )
			{
				switch( etatTuto )
				{
					/*case 1: [self tutoStepThree]; break;
					case 2: [self tutoStepThree]; break;*/
					case 3: [self tutoStepSix]; break;	
					case 5: [self tutoStepSeven]; break;
					case 6: [self tutoStepEight]; break;	
					case 7: [self endTuto]; break;	
				}
			}

			if( bGob2RightClick && counter != 28 )
				bGob2RightClick = false;
			
			if( !bPopup && !bLoadingLevel && !bSimulation && !bMenuIngame && !bHintMenuActive && !bGob2RightClick && !bGob2Flag && !bTutoOn )
			{	
				[self hotspotDisplayCompute:point];
			}
		}
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{	
	if( changeWorld || bNewGame )
		return;
	
	/*if( !bFlagCleared && !bFlagIsComputing )
		[self removeBoxFlag];*/
	
	if( !bMenuGob1 && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning && !bLoadingImage && !bFlagIsComputing )
	{
		UITouch *touch = [touches anyObject];
		CGPoint point = [touch locationInView:self];
		
		if( point.y < 5 )
			point.y = 5;
		else if( point.y > 256 )
			point.y = 256;		
		
		point.x = point.x * 320 / 480;
		point.y = point.y * 200 / 320;
		
		m.x = point.x + dist.x;
		m.y = point.y + dist.y;
		
		if( m.x < 0 )
			m.x = 0;
		else if( m.x > 313 )
			m.x = 313;
		
		if( m.y < 3 )
			m.y = 3;
		else if( m.y > 160 )
			m.y = 160;
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kInputMouseDragged], @"type",
						[NSNumber numberWithFloat:m.x], @"x",
						[NSNumber numberWithFloat:m.y], @"y",nil]];
		
		g_system->warpMouse(m.x, m.y);		
		
		if( cursorGobliiin != nil )
			cursorGobliiin.frame = CGRectMake(m.x*480/320, m.y*320/200, 40, 40);
		else
			[self cursorCurrentGobliiin:m.x coordY:m.y];	
		cursorGobliiin.hidden = NO;
				
		if( counter != 28 )
		{
			bGob2RightClick = false;
			bGob2Flag = false;
		}
		
		if( !bPopup && !bLoadingLevel && !bSimulation  && !bMenuIngame && !bHintMenuActive && !bGob2RightClick && !bGob2Flag && !bTutoOn )
		{	
			[self hotspotDisplayCompute:m];
		}
		
		bMovedCursor = true;
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{			
	if( changeWorld || bNewGame )
		return;
	
	if( !bMenuGob1 && !bPopup && bLvLLoaded && !bMenuIngame && !bHintMenuActive && !bSimulation && !bAnimBeginning && !bLoadingImage && !bFlagIsComputing )
	{
		UITouch *touch = [touches anyObject];
		CGPoint point = [touch locationInView:self];
		CGPoint touchP = point;

		if( counter == 28 && !bMovedCursor )
		{
			bGob2RightClick = false;
			bGob2Flag = false;
			//hintButton.hidden = NO;	
		}
		point.x = ((OSystem_IPHONE*)g_system)->_mouseX;
		point.y = ((OSystem_IPHONE*)g_system)->_mouseY;
		g_system->warpMouse(point.x, point.y);
		
		int ind = -1;
	
		//if icons displayed do not do events, else clear
		for( int i = 0; i < _HotspotObj.size(); ++i )
		{
			if( hotspotUsed[i] )
			{
				ind = i;
				iconsDisplayed = true;
				hotspotImgView[i].hidden = NO;
				alphaDisplayed[i] = false;
				hotspotImgView[i].alpha = 1;
			}
			else
			{
				hotspotImgView[i].hidden = YES;
				hotspotDisplayed[i] = false;
				hotspotUsed[i] = false;
				alphaDisplayed[i] = false;				
			}
		}
		
		cursorGobliiin.hidden = NO;
		
		//si pas dans hotspot, si pas animation début du jeu, si pas fin niveau, si pas fin jeu, alors
		//action deplacement rapide ou action by touch long
		if( !bPopup && !bLoadingLevel && bLvLLoaded && !bSimulation  && !bMenuIngame && !bHintMenuActive ) 
		{
			//ON NETTOIE
			/*for( int i = 0; i < 50; ++i )
			{
				hotspotImgView[i].hidden = YES;
				hotspotDisplayed[i] = false;
				hotspotUsed[i] = false;
				alphaDisplayed[i] = false;					
			}*/
			//END
			
			if( !bMovedCursor )
			{
				if( counter == 28 ) //bag
				{
					//NSLog(@"IN BAG");
					
					point.x = touchP.x * 320 / 480;
					point.y = touchP.y * 200 / 320;

					[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
									[NSNumber numberWithFloat:point.x], @"x",
									[NSNumber numberWithFloat:point.y], @"y",nil]];

				}
				else //not bag
				{
					//NSLog(@"NOT IN BAG");
					
					if( bMovementActive ) //tuto
					{				
						if( bTutoOn )
						{
							switch( etatTuto )
							{
								case 0:
								{
									if( point.x > fingusLeft && point.x < fingusLeft+22 && point.y > fingusTop && point.y < fingusTop+30 )
									{
										[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
										[self tutoStepThree];
									}
									else if( point.x > winkleLeft && point.x < winkleLeft+22 && point.y > winkleTop && point.y < winkleTop+30 )
									{
										[self tutoStepThree];										
										[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];				
									}
									else
										bMovementActive = false;
								}
								break;
							}
						}
									
						//pour regarder si un perso est pas devant un hotspot
						if( ind != -1 )
						{
							float hotspotCenterX = (_HotspotObj[ind]->left + (_HotspotObj[ind]->right - _HotspotObj[ind]->left)/2);
							float hotspotCenterY = (_HotspotObj[ind]->top  + (_HotspotObj[ind]->bottom - _HotspotObj[ind]->top)/2);		
							
							NSLog(@"hotspot : %f,%f",hotspotCenterX,hotspotCenterY);
							NSLog(@"fingus %d,%d", fingusLeft,fingusTop);
							NSLog(@"winkle %d,%d", winkleLeft,winkleTop);	
							NSLog(@"prince %d,%d", princeLeft,princeTop);	
							NSLog(@"numberGobliin %d",numberGobliin);
							
							if( (hotspotCenterX-3 > fingusLeft && hotspotCenterX+3 < fingusLeft+22 && hotspotCenterY-2 > fingusTop && hotspotCenterY+2 < fingusTop+30) || 
							   (hotspotCenterX-3 > winkleLeft && hotspotCenterX+3 < winkleLeft+22 && hotspotCenterY-2 > winkleTop && hotspotCenterY+2 < winkleTop+30) )
							{
								//NSLog(@"impossible de faire l'action");
								UIImage *mg = [UIImage imageNamed:@"inventaire2.png"];
								
								if( _warningBar == nil )
								{
									_warningBar = [[UIImageView alloc] initWithFrame:CGRectMake( 0, 50, 480, 39 )];
									_warningBar.image = mg;
									[mg release];
									[self addSubview:_warningBar];
									[_warningBar release];
								}
								else
								{
									_warningBar.hidden = NO;
								}
								
								if( _warningLabel == nil )
								{
									if( _GobLanguage == 0 )
									{
										_warningLabel = [[UILabel alloc] initWithFrame:CGRectMake( 62, 53, 480, 30 )];
										_warningLabel.text = @"Hotspots too close. Move your gobliin and try again!";
									}
									else if( _GobLanguage == 1 )
									{
										_warningLabel = [[UILabel alloc] initWithFrame:CGRectMake( 35, 53, 480, 30 )];										
										_warningLabel.text = @"Hotspot trop proche. Bougez votre gobliin et réessayez !";
									}
									_warningLabel.textColor = [UIColor whiteColor];
									_warningLabel.backgroundColor = [UIColor clearColor];
									_warningLabel.font = [UIFont fontWithName:@"Verdana" size:14];
									[_warningLabel setUserInteractionEnabled:NO];
									[self insertSubview:_warningLabel  aboveSubview:_warningBar];
									[_warningLabel release];
								}
								else
								{
									_warningLabel.hidden = NO;
								}
								
								[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
												[NSNumber numberWithFloat:point.x], @"x",
												[NSNumber numberWithFloat:point.y], @"y",nil]];	
								
								/*if( point.x >= fingusLeft && point.x <= fingusLeft+22 && point.y >= fingusTop && point.y <= fingusTop+30 )
								{
									[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
									numberGobliin = 1;
								}
								else if( point.x >= winkleLeft && point.x <= winkleLeft+22 && point.y >= winkleTop && point.y <= winkleTop+30 )
								{
									[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];
									numberGobliin = 2;
								}
								else if( (gob3State != 0 && idLvLCourant > 550 && idLvLCourant != 680 && idLvLCourant != 690 && idLvLCourant != 600 && idLvLCourant != 710) || (idLvLCourant==720 && princeLastLeft!=-1) )
								{
									if( point.x >= princeLeft && point.x <= princeLeft+22 && point.y >= princeTop && point.y <= princeTop+30 )
									{
										[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];
									}
								}
								else if( !cursorFade )
								{
									cursorFade = true;
									[self beginAlpha];					
									[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
								}*/
								
								[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(clearWarningBar) userInfo:nil repeats:NO];
							}
							else
							{
								[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
												[NSNumber numberWithFloat:point.x], @"x",
												[NSNumber numberWithFloat:point.y], @"y",nil]];
								
								if( point.x >= fingusLeft && point.x <= fingusLeft+22 && point.y >= fingusTop && point.y <= fingusTop+30 )
								{
									[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
									numberGobliin = 1;
								}
								else if( point.x >= winkleLeft && point.x <= winkleLeft+22 && point.y >= winkleTop && point.y <= winkleTop+30 )
								{
									[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];
									numberGobliin = 2;
								}
								else if( (gob3State != 0 && idLvLCourant > 550 && idLvLCourant != 680 && idLvLCourant != 690 && idLvLCourant != 600 && idLvLCourant != 710) || (idLvLCourant==720 && princeLastLeft!=-1) )
								{
									if( point.x >= princeLeft && point.x <= princeLeft+22 && point.y >= princeTop && point.y <= princeTop+30 )
									{
										[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];
									}
								}
								else if( !cursorFade )
								{
									cursorFade = true;
									[self beginAlpha];					
									[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(doClear) userInfo:nil repeats:NO];
								}
							}
						}
						else
						{
							[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
											[NSNumber numberWithFloat:point.x], @"x",
											[NSNumber numberWithFloat:point.y], @"y",nil]];	
							
							if( point.x >= fingusLeft && point.x <= fingusLeft+22 && point.y >= fingusTop && point.y <= fingusTop+30 )
							{
								[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
								numberGobliin = 1;								
							}
							else if( point.x >= winkleLeft && point.x <= winkleLeft+22 && point.y >= winkleTop && point.y <= winkleTop+30 )
							{
								[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];
								numberGobliin = 2;
							}
							else if( (gob3State != 0 && idLvLCourant > 550 && idLvLCourant != 680 && idLvLCourant != 690 && idLvLCourant != 600 && idLvLCourant != 710) || (idLvLCourant==720 && princeLastLeft!=-1) )
							{
								if( point.x >= princeLeft && point.x <= princeLeft+22 && point.y >= princeTop && point.y <= princeTop+30 )
								{
									[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];
								}
							}
						}
					}
				}
			}
		}	
		iconsDisplayed = false;
	}
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
	NSLog(@"iPhoneView touchesCancelled");	
}

- (void)clearButtons
{	
	//[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
	cursorGobliiin.frame = CGRectMake(((OSystem_IPHONE*)g_system)->_mouseX *480/320,((OSystem_IPHONE*)g_system)->_mouseY *320/200, 40, 40);
	cursorGobliiin.hidden = NO;
	
	//-----------------------
	//we re quitting this mode
	iconsDisplayed = false;
	bStepIconOn = false;
	
	//we clear the view, and these are set to false
	for( int i = 0; i < 50; ++i )
	{
		alphaDisplayed[i] = false;		
		hotspotImgView[i].hidden = YES;
		hotspotDisplayed[i] = false;
		hotspotUsed[i] = false;
	}
	//------------------------
	//END
}

std::vector< NSString* > _flagPlaces;
bool _getFlagPlaces = false;

-(void)flag
{
	NSLog(@"iPhoneViewController flag");	
	
	g_system->warpMouse( 250, 0 );
	
	//move the goblin
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:0], @"x",
					[NSNumber numberWithFloat:0], @"y",nil]];	
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
					[NSNumber numberWithFloat:0], @"x",
					[NSNumber numberWithFloat:0], @"y",nil]];	
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:0], @"x",
					[NSNumber numberWithFloat:0], @"y",nil]];	
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:0], @"x",
					[NSNumber numberWithFloat:0], @"y",nil]];	
}

- (void) removePlacesButtons
{
	NSLog(@"removePlaceButtons %d", _flagPlaces.size() );
	for( unsigned int i = 0; i < _flagPlaces.size(); ++i )
	{
		if( _flagLabel[i] != nil )
		{
			_flagLabel[i].hidden = YES;			
		}
		
		if( _flagText[i] != nil )
		{
			_flagText[i].hidden = YES;
		}
	}	
	
	if( _flagBackground != nil && _flagBackground.superview != nil )
	{
		_flagBackground.hidden = YES;
	}
	
	_flagPlaces.clear();
	_getFlagPlaces = false;
	bFlagCleared = true;
}

- (void) removeBoxFlag
{
	//switch to fingus pr travel
	float xTouch = fingusLeft + 10;
	float yTouch = fingusTop + 10;
	if( yTouch <= 10 ) yTouch = 10;
	g_system->warpMouse( xTouch, yTouch );
	
	[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
					[NSNumber numberWithFloat:xTouch], @"x",
					[NSNumber numberWithFloat:yTouch], @"y",nil]];
	
	[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(removePlacesButtons) userInfo:nil repeats:NO];
}

- (void) placesClicked:(id)sender
{
	NSLog(@"placesClicked:");
	
	for( int i = 0; i < _flagPlaces.size(); ++i )
	{
		if( _flagLabel[i] == sender  )
		{	
			g_system->warpMouse( 270, (i*12) + 12 );
			//[self cursorCurrentGobliiin:270 coordY:(i*12) + 12];
			
			[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
							[NSNumber numberWithFloat:0], @"x",
							[NSNumber numberWithFloat:0], @"y",nil]];
			
			[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(removePlacesButtons) userInfo:nil repeats:NO];
		}
	}
}

-(int)torgba:(uint16_t)i
{
	uint16_t temp = i; // swap bytes for Big-Endian (data[0] = 0xRRRRRGGG, data[1] = 0xGGGBBBBB) bitmap
	uint8_t red, green, blue; //these only need to be one byte each
	red = (temp >> 11) & 0x1F;
	green = (temp >> 5) & 0x3F;
	blue = (temp & 0x001F);    
	red = (red << 3) | (red >> 2);
	green = (green << 2) | (green >> 4);
	blue = (blue << 3) | (blue >> 2);    
	
	return (int)((red << 24) | (green << 16) | (blue << 8) | 0xFF);
}

- (void) computeFlagPlaces
{
	if( _flagPlaces.size() <= 0 ) return;	
	_flagPlaces.pop_back();
	if( _flagPlaces.size() <= 0 ) return;
	
	UIImage *backgroundStretch = [[UIImage imageNamed:@"flagBackground.png"] stretchableImageWithLeftCapWidth:6.0f topCapHeight:6.0f];
	
	if( _flagBackground == nil )
	{
		_flagBackground = [[UIImageView alloc] initWithFrame:CGRectMake( 335, 2, 145, _flagPlaces.size()*52 )];
		_flagBackground.autoresizingMask = UIViewAutoresizingFlexibleWidth;
		_flagBackground.image = backgroundStretch;
		[backgroundStretch release];
		[self addSubview:_flagBackground];
		[_flagBackground release];
	}
	else
	{
		_flagBackground.frame = CGRectMake( 335, 2, 145, _flagPlaces.size()*52 );
		_flagBackground.hidden = NO;
	}
	
	NSLog(@"computeFlagPlaces %d",_flagPlaces.size());
	
	for( unsigned int i = 0; i < _flagPlaces.size(); ++i )
	{
		NSLog(@"%@",_flagPlaces[i]);
		
		if( _flagLabel[i] == nil )
		{
			_flagLabel[i] = [[UIButton buttonWithType:UIButtonTypeCustom] retain];
			_flagLabel[i].frame = CGRectMake( 345, i*45 + 8, 128, 35 );	
			[_flagLabel[i] setImage:[UIImage imageNamed:@"teleport.png"] forState:UIControlStateNormal];		
			[_flagLabel[i] addTarget:self action:@selector(placesClicked:) forControlEvents:UIControlEventTouchUpInside];
			[self addSubview:_flagLabel[i]];
			[_flagLabel[i] release];
		}
		else
		{
			_flagLabel[i].frame = CGRectMake( 345, i*45 + 8, 128, 35 );	
			_flagLabel[i].hidden = NO;
		}
		
		if( _flagText[i] == nil )
		{
			_flagText[i] = [[UILabel alloc] initWithFrame:CGRectMake( 365, i*45 + 11, 100, 30 )];
			_flagText[i].textColor = [UIColor blackColor];
			_flagText[i].backgroundColor = [UIColor clearColor];
			_flagText[i].font = [UIFont fontWithName:@"Verdana" size:16];
			_flagText[i].text = _flagPlaces[i];
			[_flagText[i] setUserInteractionEnabled:NO];
			[self insertSubview:_flagText[i] aboveSubview:_flagLabel[i]];
			[_flagText[i] release];
		}
		else
		{
			_flagText[i].frame = CGRectMake( 365, i*45 + 11, 100, 30 );	
			_flagText[i].hidden = NO;
			_flagText[i].text = _flagPlaces[i];
		}
	}
	
	bFlagCleared = false;
}

-(void)removeInvScreen
{
	/*if( !bBackground )
		[self computeFlagPlaces];*/
	
	if( invView != nil && invView.superview != nil )
	 {
		 [invView removeFromSuperview];
		 invView = nil;
	 }	
	[flagAct stopAnimating];
	bFlagIsComputing = false;	
}

-(void) printInvScreen
{
	int a[320*200] = { 0 };
	for(int i = 0; i <320*200; i++)
		a[i] = [self torgba:_fullscreen[i]];
	
	// make data provider with data.
	CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, &a, 320*200*4, NULL);
	
	// prep the ingredients
	int bitsPerComponent = 8;
	int bitsPerPixel = 32;
	int bytesPerRow = 4 * 320;
	
	CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
	CGBitmapInfo bitmapInfo = kCGBitmapByteOrder32Little;
	CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;
	
	// make the cgimage
	CGImageRef imageRef = CGImageCreate(320, 200, bitsPerComponent, bitsPerPixel, bytesPerRow, colorSpaceRef, bitmapInfo, provider, NULL, NO, renderingIntent);
	
	// then make the uiimage from that
	invScreen = [UIImage imageWithCGImage:imageRef];	

	docsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
	path_inv = [docsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"inv.png"]];	
	
	NSFileManager* fileManager = [NSFileManager defaultManager];
	[fileManager removeItemAtPath:path_inv error:NULL];
	
	NSData *imageData = [NSData dataWithData:UIImagePNGRepresentation(invScreen)];
	[imageData writeToFile:path_inv atomically:YES];
	
	invScreen = [UIImage imageWithContentsOfFile:path_inv];
	
	CGSize size = CGSizeMake(320, 200);
	UIGraphicsBeginImageContext( size );
	[invScreen drawInRect:CGRectMake(0,0,size.width,size.height)];
	invScreen = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
	CGDataProviderRelease(provider);
	CGColorSpaceRelease(colorSpaceRef);
	CGImageRelease(imageRef);
	
	invView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 480, 320)];			
	[invView setImage:invScreen];			
	[self addSubview:invView];	
	[self insertSubview:menuButton aboveSubview:invView];
	[self insertSubview:hintButton aboveSubview:invView];		
	[self insertSubview:choiceButton aboveSubview:invView];	
	[self insertSubview:rightClickButton aboveSubview:invView];
	[self insertSubview:gobliinButton aboveSubview:invView];	
	[self insertSubview:cursorGobliiin aboveSubview:invView];
	[self insertSubview:openMenuBar aboveSubview:invView];
	[self insertSubview:closeMenuBar aboveSubview:invView];	
	[self insertSubview:menuBar aboveSubview:invView];		
	[self insertSubview:flagAct aboveSubview:invView];		
}

- (void) computeFlag
{
	if( bGob2Action == 1 )
	{			
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kTempo], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];
		
		[NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(flag) userInfo:nil repeats:NO];
		[NSTimer scheduledTimerWithTimeInterval:4 target:self selector:@selector(removeInvScreen) userInfo:nil repeats:NO];		
	}
	else if( 28 == counter )
	{
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];
		
		[NSTimer scheduledTimerWithTimeInterval:0.65 target:self selector:@selector(flag) userInfo:nil repeats:NO];	
		[NSTimer scheduledTimerWithTimeInterval:3.5 target:self selector:@selector(removeInvScreen) userInfo:nil repeats:NO];			
	}
	else
	{
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];	
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(flag) userInfo:nil repeats:NO];
		[NSTimer scheduledTimerWithTimeInterval:2.25 target:self selector:@selector(removeInvScreen) userInfo:nil repeats:NO];				
	}	
}

-(IBAction) choiceButtonClicked:(id)sender
{
	NSLog(@"iPhoneViewController choiceButtonClicked");	
	[self hideMenuBar];
	
	[self doClear];
	
	if( !bMenuIngame && !bHintMenuActive && !bGob2Flag && !bTutoOn )
	{	
		//switch to fingus pr travel
		float xTouch = fingusLeft + 12;
		float yTouch = fingusTop + 20;
		if( yTouch <= 10 ) yTouch = fingusTop+20;
		g_system->warpMouse( xTouch, yTouch );
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
										   [NSNumber numberWithFloat:xTouch], @"x",
										   [NSNumber numberWithFloat:yTouch], @"y",nil]];
		[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
		
		//recuperation du nom des voyages
		//_getFlagPlaces = false;
		//_flagPlaces.clear();
		
		//activity indicator	
		flagAct.hidden = NO;		
		[flagAct startAnimating];
		bFlagIsComputing = true;

		[self printInvScreen];
		
		if( !bGob2Flag )
			bGob2Flag = true;

		//on nettoie pour changer // TO DO - changer en fonction du goblin
		[self clearButtons];
		
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(computeFlag) userInfo:nil repeats:NO];
	}
}

- (IBAction) rightButtonClicked:(id)sender
{
	NSLog(@"iPhoneViewController rightButtonClicked");	
	[self hideMenuBar];
	
	[self doClear];
	
	//[self removeObjectsToGetButtons];
	
	if( !bFlagCleared && !bFlagIsComputing )
		return; //[self removeBoxFlag];
	
	if( bFlagIsComputing )
		return;
	
	if( !bMenuIngame && !bHintMenuActive && !bTutoOn )
	{
		if( !bGob2RightClick )
			bGob2RightClick = true;

		[self clearButtons];
		
		[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonRightDown], @"type",
						[NSNumber numberWithFloat:0], @"x",
						[NSNumber numberWithFloat:0], @"y",nil]];	
		
		if( counter == 28 )
			bGob2RightClick = false;	
	}
}

-(void)resetPosCursor
{
	((OSystem_IPHONE*)g_system)->_mouseX = xPosCur;
	((OSystem_IPHONE*)g_system)->_mouseY = yPosCur;
	cursorGobliiin.frame = CGRectMake(((OSystem_IPHONE*)g_system)->_mouseX *480/320,((OSystem_IPHONE*)g_system)->_mouseY *320/200, 40, 40);	
}

- (void) clearWarningSuperposBar
{
	_warningSuperposBar.hidden = YES;
	_warningSuperposLabel.hidden = YES;	
}

- (void) warningSuperposition
{
	UIImage *mg = [UIImage imageNamed:@"inventaire2.png"];
	
	if( _warningSuperposBar == nil )
	{
		_warningSuperposBar = [[UIImageView alloc] initWithFrame:CGRectMake( 0, 50, 480, 39 )];
		_warningSuperposBar.image = mg;
		[mg release];
		[self addSubview:_warningSuperposBar];
		[_warningSuperposBar release];
	}
	else
	{
		_warningSuperposBar.hidden = NO;
	}
	
	if( _warningSuperposLabel == nil )
	{
		if( _GobLanguage == 0 )
		{
			_warningSuperposLabel = [[UILabel alloc] initWithFrame:CGRectMake( 23, 53, 480, 30 )];
			_warningSuperposLabel.text = @"Your gobliins are too close! Move one of them and try again!";
		}
		else if( _GobLanguage == 1 )
		{
			_warningSuperposLabel = [[UILabel alloc] initWithFrame:CGRectMake( 15, 53, 480, 30 )];										
			_warningSuperposLabel.text = @"Vos Gobliins sont trop proches. Séparez-les et recommencez !";
		}
		_warningSuperposLabel.textColor = [UIColor whiteColor];
		_warningSuperposLabel.backgroundColor = [UIColor clearColor];
		_warningSuperposLabel.font = [UIFont fontWithName:@"Verdana" size:14];
		[_warningSuperposLabel setUserInteractionEnabled:NO];
		[self insertSubview:_warningSuperposLabel  aboveSubview:_warningSuperposBar];
		[_warningSuperposLabel release];
	}
	else
	{
		_warningSuperposLabel.hidden = NO;
	}
	
	[NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(clearWarningSuperposBar) userInfo:nil repeats:NO]; 
}

bool changeGobliin = false;

-(void) gobliinButtonClicked:(id)sender
{
	NSLog(@"iPhoneViewController gobliinButtonClicked");	
	
	if( bFlagIsComputing )
		return;
	
	[self hideMenuBar];
	
	[self doClear];	
	
	if( !bTutoOn )
	{
		if( !bMenuIngame && !bHintMenuActive )
		{			
			//on nettoie pour changer // TO DO - changer en fonction du goblin
			[self clearButtons];
			
			//xPosCur = ((OSystem_IPHONE*)g_system)->_mouseX;
			//yPosCur = ((OSystem_IPHONE*)g_system)->_mouseY;
			
			//[self cursorCurrentGobliiin:((OSystem_IPHONE*)g_system)->_mouseX coordY:((OSystem_IPHONE*)g_system)->_mouseY];
			
			changeGobliin = true;
			
			float xTouch, yTouch;
			bool superpos = false;
			
			if( gob3State == 0 || idLvLCourant == 550 || idLvLCourant == 680 || idLvLCourant == 690 || idLvLCourant == 600 || idLvLCourant == 710 || ((idLvLCourant==720 || idLvLCourant==560) && princeLastLeft==-1) )
			{
				if( numberGobliin == 2 )
				{
					xTouch = fingusLeft + 10;
					yTouch = fingusTop + 10;
					
					if( xTouch >= winkleLeft && xTouch <= winkleLeft+22 && yTouch >= winkleTop && yTouch <= winkleTop+30 )
					{
						NSLog(@"superposition A %d,%d",fingusLeft,winkleLeft);
						superpos = true;
						[self warningSuperposition];
					}
					else
					{
						[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
					}
				}
				else if( numberGobliin == 1 )
				{
					xTouch = winkleLeft + 10;
					yTouch = winkleTop + 10;	
					
					if( xTouch >= fingusLeft && xTouch <= fingusLeft+22 && yTouch >= fingusTop && yTouch <= fingusTop+30 )
					{
						NSLog(@"superposition B %d,%d",fingusLeft,winkleLeft);
						superpos = true;	
						[self warningSuperposition];
					}
					else
					{
						[gobliinButton setImage:[UIImage imageNamed:@"winkle.png"] forState:UIControlStateNormal];
					}
				}
			}
			else
			{
				if( numberGobliin == 3 )
				{
					xTouch = fingusLeft + 10;
					yTouch = fingusTop + 10;
					
					if( xTouch >= princeLeft && xTouch <= princeLeft+22 && yTouch >= princeTop && yTouch <= princeTop+30 )
					{
						NSLog(@"superposition A %d,%d",fingusLeft,princeLeft);
						superpos = true;
						[self warningSuperposition];
					}
					else
					{
						[gobliinButton setImage:[UIImage imageNamed:@"fingus.png"] forState:UIControlStateNormal];
					}
				}
				else if( numberGobliin == 1 )
				{
					xTouch = winkleLeft + 10;
					yTouch = winkleTop + 10;		
					
					if( xTouch >= fingusLeft && xTouch <= fingusLeft+22 && yTouch >= fingusTop && yTouch <= fingusTop+30 )
					{
						NSLog(@"superposition B %d,%d",fingusLeft,winkleLeft);
						superpos = true;	
						[self warningSuperposition];
					}
					else
					{
						[gobliinButton setImage:[UIImage imageNamed:@"winkle_p.png"] forState:UIControlStateNormal];
					}
				}	
				else if( numberGobliin == 2 )
				{
					xTouch = princeLeft + 10;
					yTouch = princeTop + 10;		
					
					if( xTouch >= winkleLeft && xTouch <= winkleLeft+22 && yTouch >= winkleTop && yTouch <= winkleTop+30 )
					{
						NSLog(@"superposition C %d,%d",princeLeft,winkleLeft);
						superpos = true;
						[self warningSuperposition];
					}
					else
					{
						[gobliinButton setImage:[UIImage imageNamed:@"prince_f.png"] forState:UIControlStateNormal];
					}
				}	
			}
			
			if( !superpos )
			{
				if( yTouch <= 10 ) yTouch = 10;
				g_system->warpMouse( xTouch , yTouch );	
				
				cursorGobliiin.frame = CGRectMake(xTouch *480/320,yTouch *320/200, 40, 40);	
			
				[self addEvent:[[NSDictionary alloc] initWithObjectsAndKeys:[NSNumber numberWithInt:kButtonLeftDown], @"type",
								[NSNumber numberWithFloat:xTouch], @"x",
								[NSNumber numberWithFloat:yTouch], @"y",nil]];	
			}
			
			//[NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(resetPosCursor) userInfo:nil repeats:NO];
		}
	}
}

- (void) showMenuBar
{
	[self doClear];
	
	if( !bMenuIngame && !bHintMenuActive && !bTutoOn )
	{
		menuButton.hidden = NO;	
		hintButton.hidden = NO;
		choiceButton.hidden = NO;		
		
		openMenuBar.hidden = YES;
		closeMenuBar.hidden = NO;
		menuBar.hidden = NO;	
		
		if( idLvLCourant == 670 || idLvLCourant == 680 || idLvLCourant == 690 || idLvLCourant == 710 || idLvLCourant == 720 )
			choiceButton.enabled = NO;
		else
			choiceButton.enabled = YES;
	}
}

- (void) hideMenuBar
{	
	if( !bTutoOn )
	{	
		menuButton.hidden = YES;	
		hintButton.hidden = YES;
		choiceButton.hidden = YES;
		
		openMenuBar.hidden = NO;	
		closeMenuBar.hidden = YES;
		menuBar.hidden = YES;	
	}
}

- (void) openMenuBarClicked:(id)sender
{
	NSLog(@"iPhoneView openMenuBarClicked:");
	if( counter == 28 )
		return;
	[self showMenuBar];
}

- (void) closeMenuBarClicked:(id)sender
{
	NSLog(@"iPhoneView closeMenuBarClicked:");	
	[self hideMenuBar];
}

@end

//---C functions
bool getLocalMouseCoords(CGPoint *point)
{
	return true;
	//printf("point %d;%d - screenRect %d;%d w:%d h:%d\n",point->x, point->y, _screenRect.origin.x,  _screenRect.origin.y, _screenRect.size.width, _screenRect.size.height);
	if (point->x < _screenRect.origin.x || point->x >= _screenRect.origin.x + _screenRect.size.width ||
		point->y < _screenRect.origin.y || point->y >= _screenRect.origin.y + _screenRect.size.height)
	{
		return false;
	}
	
	point->x = (point->x - _screenRect.origin.x) / _screenRect.size.width;
	point->y = (point->y - _screenRect.origin.y) / _screenRect.size.height;
	
	return true;
}

uint getSizeNextPOT(uint size) 
{
    if ((size & (size - 1)) || !size)
	{
        int log = 0;
		
        while (size >>= 1)
            ++log;
		
        size = (2 << log);
    }
	
    return size;
}
//----

void iPhone_updateScreen()
{
	if (!_needsScreenUpdate) 
	{
		_needsScreenUpdate = 1;
		[sharedInstance performSelectorOnMainThread:@selector(updateSurface) withObject:nil waitUntilDone: NO];
	}
}

void iPhone_updateScreenRect(unsigned short* screen, int x1, int y1, int x2, int y2) 
{
	//[_lock lock];

	int y;
	for (y = y1; y < y2; ++y) 
	{
		memcpy(&_textureBuffer[(y * _textureWidth + x1 )* 2], &screen[y * _width + x1], (x2 - x1) * 2);
	}

	//[_lock unlock];
}


void iPhone_initSurface(int width, int height)
{
	_width = width;
	_height = height;
	[sharedInstance performSelectorOnMainThread:@selector(initSurface) withObject:nil waitUntilDone: YES];
}

bool iPhone_fetchEvent(int *outEvent, float *outX, float *outY)
{
	id event = [sharedInstance getEvent];
	if (event == nil) 
	{
		return false;
	}

	id type = [event objectForKey:@"type"];

	if (type == nil)
	{
		printf("fetchEvent says: No type!\n");
		return false;
	}

	*outEvent = [type intValue];
	*outX = [[event objectForKey:@"x"] floatValue];
	*outY = [[event objectForKey:@"y"] floatValue];
	return true;
}

const char* iPhone_getDocumentsDir()
{
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return [documentsDirectory UTF8String];
}

const char* iPhone_getBundleDir() 
{
	NSString *bundleDirectory = [[NSBundle mainBundle] bundlePath];	
	return [bundleDirectory UTF8String];
}

